searchData={"items":[{"type":"module","title":"AvroSchema","doc":"# AvroSchema\n\n---\n\n[![CircleCI](https://circleci.com/gh/cogini/avro_schema.svg?style=svg)](https://app.circleci.com/github/cogini/avro_schema/pipelines)\n\nThis is a library for working with [Avro](https://avro.apache.org/)\nschemas and the [Confluent® Schema Registry](https://www.confluent.io/confluent-schema-registry),\nprimarily focused on working with [Kafka](https://kafka.apache.org/) streams.\nIt relies on [erlavro](https://github.com/klarna/erlavro) for encoding and\ndecoding data and [confluent_schema_registry](https://github.com/cogini/confluent_schema_registry)\nto look up schemas using the [Schema Registry API](https://docs.confluent.io/current/schema-registry/develop/api.html).\n\nIts primary value is that it caches schemas for performance and to allow\nprograms to work independently of the Schema Registry being available.\nIt also has a consistent set of functions to manage schema tags, look up\nschemas from the Schema Registry or files, and encode/decode data.\n\nMuch thanks to Klarna for [Avlizer](https://github.com/klarna/avlizer), which\nprovides similar functionality to this library in Erlang,\n[erlavro](https://github.com/klarna/erlavro) for Avro, and\n[brod](https://github.com/klarna/brod) for dealing with Kafka.","ref":"AvroSchema.html"},{"type":"module","title":"Installation - AvroSchema","doc":"Add the package to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:avro_schema, \"~> 0.1.0\"}]\nend\n```\n\nThen run `mix deps.get` to fetch the new dependency.\n\nDocumentation is on [HexDocs](https://hexdocs.pm/avro_schema).\nTo generate a local copy, run `mix docs`.","ref":"AvroSchema.html#module-installation"},{"type":"module","title":"Starting - AvroSchema","doc":"Add the cache GenServer to your application's supervision tree:\n\n```elixir\ndef start(_type, _args) do\n  cache_dir = Application.get_env(:yourapp, :cache_dir, \"/tmp\")\n\n  children = [\n    {AvroSchema, [cache_dir: cache_dir]},\n  ]\n\n  opts = [strategy: :one_for_one, name: LogElasticsearch.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"AvroSchema.html#module-starting"},{"type":"module","title":"Overview - AvroSchema","doc":"When using Kafka, producers and consumers are separated, and schemas may evolve\nover time. It is common for producers to tag data indicating the schema that\nwas used to encode it. Consumers can then look up the corresponding schema\nversion and use it decode the data.\n\nThis library supports two tagging formats, [Confluent wire format](https://docs.confluent.io/current/schema-registry/serializer-formatter.html#wire-format),\nand [Avro single object encoding](https://avro.apache.org/docs/1.8.2/spec.html#single_object_encoding_spec).","ref":"AvroSchema.html#module-overview"},{"type":"module","title":"Confluent wire format - AvroSchema","doc":"With the [Confluent wire format](https://docs.confluent.io/current/schema-registry/serializer-formatter.html#wire-format),\nAvro binary encoded objects are prefixed with a five-byte tag.\n\nThe first byte indicates the Confluent serialization format version number,\ncurrently always 0. The following four bytes encode the integer schema ID as\nreturned from the Schema Registry in network byte order.","ref":"AvroSchema.html#module-confluent-wire-format"},{"type":"module","title":"Avro single-object encoding - AvroSchema","doc":"When used without a schema registry, it's common to prefix binary data with a\nhash of the schema that created it. In the past, that might be something like\nMD5.\n\nThe Avro \"Single-object encoding\" formalizes this, prefixing Avro binary data\nwith a two-byte marker, C3 01, to show that the message is Avro and uses this\nsingle-record format (version 1). That is followed by the 8-byte little-endian\n[CRC-64-AVRO](https://avro.apache.org/docs/1.8.2/spec.html#schema_fingerprints)\nfingerprint of the object's schema.\n\nThe CRC64 algorithm is uncommon, but used because it is relatively short,\nwhile still being good enough to detect collisions. The fingerprint function is\nimplemented in `fingerprint_schema/1`.","ref":"AvroSchema.html#module-avro-single-object-encoding"},{"type":"module","title":"Schema Registry - AvroSchema","doc":"In a relatively static system, it's not too hard to exchange schema files\nbetween producers and consumers. When things are changing more frequently, it\ncan be difficult to keep files up to date. It's also easy for insignificant\ndifferences such as whitespace to result in different schema hashes.\n\nThe Schema Registry solves this by providing a centralized service which\nproducers and consumers can call to get a unique identifier for a schema\nversion. Producers register a schema with the service and get an id.\nConsumers look up the id to get the schema.\n\nThe Schema Registry also does validation on new schemas to ensure that they\nmeet a backwards compatibility policy for the organization.\nThis helps you to [evolve schemas](https://docs.confluent.io/current/schema-registry/avro.html)\nover time and deploy them without breaking running applications.\n\nThe disadvantage of the Schema Registry is that it can be a single point\nof failure. Different schema registries will in general assign a different\nnumeric id to the same schema.\n\nThis library provides functions to register schemas with the Schema Registry\nand look them up by id. It caches the results in RAM (ETS) for performance,\nand optionally also on disk (DETS). This gives good performance and allows\nprograms to work without needing to communicate with the Schema Registry.\nOnce read, the numeric IDs never change, so it's safe to cache them indefinitely.\n\nThe library also has support for managing schemas from files. It can add files\nto the cache by fingerprint, registering the same schema under multiple\nfingerprints, i.e. the raw JSON, a version in\n[Parsing Canonical Form](https://avro.apache.org/docs/current/spec.html#Parsing+Canonical+Form+for+Schemas)\nand with whitespace stripped out. You can also manually register aliases for the\nname and fingerprint to handle legacy data.","ref":"AvroSchema.html#module-schema-registry"},{"type":"module","title":"Kafka producer example - AvroSchema","doc":"A Kafka producer program needs to be able to encode the data with an Avro\nschema and tag it with the schema ID or fingerprint. It may store the\nschema in the code or read it from a file, or it may look it up from the Schema\nRegistry using the subject.\n\nThe subject is a registered name which identifies the type of data.\nThere are are several [standard strategies](https://docs.confluent.io/current/schema-registry/serializer-formatter.html#subject-name-strategy)\nused by Confluent in their Kafka libraries.\n\n* `TopicNameStrategy`, the default, registers the schema based on the Kafka\n  topic name, implicitly requiring that all messages use the same schema.\n\n* `RecordNameStrategy` names the schema using the record type, allowing\n  a single topic to have multiple different types of data or multiple topics\n  to have the same type of data.\n\n  In an Avro schema the \"[full name](https://avro.apache.org/docs/1.8.2/spec.html#names)\", is\n  a namespace-qualified name for the record, e.g. `com.example.X`. In the schema, it is\n  the `name` field.\n\n* `TopicRecordNameStrategy` names the schema using a combination of topic and record.\n\nWith the subject, the producer can call the Schema Registry to get the ID\nmatching the Avro schema:\n\n```elixir\niex> schema_json = \"{\\\"name\\\":\\\"test\\\",\\\"type\\\":\\\"record\\\",\\\"fields\\\":[{\\\"name\\\":\\\"field1\\\",\\\"type\\\":\\\"string\\\"},{\\\"name\\\":\\\"field2\\\",\\\"type\\\":\\\"int\\\"}]}\"\niex> subject = \"test\"\niex> {:ok, ref} = AvroSchema.register_schema(subject, schema_json)\n{:ok, 21}\n```\n\nIf the schema has already been registered, then the Schema Registry will\nreturn the current id. If you are registering a new version of the schema, then\nthe Schema Registry will first check if it is compatible with the old one.\nDepending on the compatibility rules, it may reject the schema.\n\nThe producer next needs to get an encoder for the schema.\n\nThe encoder is a function that takes Avro key/value data and encodes\nit to a binary.\n\n```elixir\niex> {:ok, encoder} = AvroSchema.make_encoder(schema_json)\n{:ok, #Function<2.110795165/1 in :avro.make_simple_encoder/2>}\n```\n\nNext we encode some data:\n\n```elixir\niex> data = %{field1: \"hello\", field2: 100}\niex> encoded = AvroSchema.encode!(data, encoder)\n[['\\n', \"hello\"], [200, 1]]\n```\n\nFinally, we tag the data:\n\n```elixir\niex> tagged_confluent = AvroSchema.tag(encoded, 21)\n[<<0, 0, 0, 0, 21>>, [['\\n', \"hello\"], [200, 1]]]\n```\n\nIf you are using files, the process is similar. First\ncreate a fingerprint for the schema:\n\n```elixir\niex> fp = AvroSchema.fingerprint_schema(schema_json)\n<<172, 194, 58, 14, 16, 237, 158, 12>>\n```\n\nNext tag the data:\n\n```elixir\niex> tagged_avro = AvroSchema.tag(encoded, fp)\n[\n  <<195, 1>>,\n  <<172, 194, 58, 14, 16, 237, 158, 12>>,\n  [['\\n', \"hello\"], [200, 1]]\n]\n```\n\nNow you can send the data to Kafka.","ref":"AvroSchema.html#module-kafka-producer-example"},{"type":"module","title":"Kafka consumer example - AvroSchema","doc":"The process for a consumer is similar.\n\nReceive the data and get the registration id in Confluent format:\n\n```elixir\niex> tagged_confluent = IO.iodata_to_binary(AvroSchema.tag(encoded, 21))\n<<0, 0, 0, 0, 21, 10, 104, 101, 108, 108, 111, 200, 1>>\n\niex> {:ok, {{:confluent, regid}, bin}} = AvroSchema.untag(tagged_confluent)\n{:ok, {{:confluent, 21}, <<10, 104, 101, 108, 108, 111, 200, 1>>}}\n```\n\nGet the schema from the Schema Registry:\n\n```elixir\niex> {:ok, schema} = AvroSchema.get_schema(regid)\n{:ok,\n {:avro_record_type, \"test\", \"\", \"\", [],\n  [\n    {:avro_record_field, \"field1\", \"\", {:avro_primitive_type, \"string\", []},\n     :undefined, :ascending, []},\n    {:avro_record_field, \"field2\", \"\", {:avro_primitive_type, \"int\", []},\n     :undefined, :ascending, []}\n  ], \"test\", []}}\n```\n\nCreate a decoder and decode the data:\n\n```elixir\niex> {:ok, decoder} = AvroSchema.make_decoder(schema)\n{:ok, #Function<4.110795165/1 in :avro.make_simple_decoder/2>}\n\niex> decoded = AvroSchema.decode!(bin, decoder)\n%{\"field1\" => \"hello\", \"field2\" => 100}\n```\n\nThe process is similar with a fingerprint.  In this case, we get the schema\nfrom files and register it in the cache using the schema name and fingerprints. There\nis more than one fingerprint because we register it with the raw schema from\nthe file and the normalized JSON for better interop.\n\n```elixir\niex> {:ok, files} = AvroSchema.get_schema_files(\"test/schemas\")\n{:ok, [\"test/schemas/test.avsc\"]}\n\niex> for file <- files, do: AvroSchema.cache_schema_file(file)\n[\n  ok: [\n    {\"test\", <<172, 194, 58, 14, 16, 237, 158, 12>>},\n    {\"test\", <<194, 132, 80, 199, 36, 146, 103, 147>>}\n  ]\n]\n```\n\nTo decode, separate the fingerprint from the data:\n\n```elixir\niex> tagged_avro = IO.iodata_to_binary(AvroSchema.tag(encoded, fp))\n<<195, 1, 172, 194, 58, 14, 16, 237, 158, 12, 10, 104, 101, 108, 108, 111, 200,\n  1>>\n\niex> {:ok, {{:avro, fp}, bin}} = AvroSchema.untag(tagged_avro)\n{:ok, {{:avro, <<172, 194, 58, 14, 16, 237, 158, 12>>},\n  <<10, 104, 101, 108, 108, 111, 200, 1>>}}\n```\n\nGet the decoder and decode the data:\n\n```elixir\niex> {:ok, schema} = AvroSchema.get_schema({\"test\", fp})\n{:ok,\n {:avro_record_type, \"test\", \"\", \"\", [],\n  [\n    {:avro_record_field, \"field1\", \"\", {:avro_primitive_type, \"string\", []},\n     :undefined, :ascending, []},\n    {:avro_record_field, \"field2\", \"\", {:avro_primitive_type, \"int\", []},\n     :undefined, :ascending, []}\n  ], \"test\", []}}\n```\n\nDecoding works the same as with the Schema Registry:\n\n```elixir\niex> {:ok, decoder} = AvroSchema.make_decoder(schema)\n{:ok, #Function<4.110795165/1 in :avro.make_simple_decoder/2>}\n\niex> decoded = AvroSchema.decode!(bin, decoder)\n%{\"field1\" => \"hello\", \"field2\" => 100}\n```","ref":"AvroSchema.html#module-kafka-consumer-example"},{"type":"module","title":"Performance - AvroSchema","doc":"For best performance, save the encoder or decoder in your process\nstate to avoid the overhead of looking it up for each message.\n\nAn in-memory ETS cache maps the integer registry ID or name + fingerprint\nto the corresponding schema and decoder. It also allows lookups using name and\nfingerprint as a key.\n\nThe fingerprint is CRC64 by default. You can also register a name with your own\nfingerprint.\n\nThis library also allows consumers to look up the schema on demand from the\nSchema Registry using the name + fingerprint as the registry subject name.\n\nThis library can optionally persist the cache data on disk using DETS,\nallowing programs to work without continuous access to the Schema Registry.\n\nPrograms which use Kafka may process high message volumes, so efficiency\nis important. They generally use multiple processes, typically one per\ntopic partition or more. On startup, each process may simultaneously attempt to\nlook up schemas.\n\nThe cache lookup runs in the caller's process, so it can run in parallel.\nIf there is a cache miss, then it calls the GenServer to update the cache.\nThis has the effect of serializing requests, ensuring that only one runs\nat a time. See https://www.cogini.com/blog/avoiding-genserver-bottlenecks/ for\ndiscussion.\n\nIn order to improve interoperability, the schema should be put into standard form.\n\nIt might also call the schema registry to get the schema for a given subject:\n\n```elixir\niex> ConfluentSchemaRegistry.get_schema(client, \"test\")\n{:ok,\n%{\n \"id\" => 21,\n \"schema\" => \"{\\\"type\\\":\\\"record\\\",\\\"name\\\":\\\"test\\\",\\\"fields\\\":[{\\\"name\\\":\\\"field1\\\",\\\"type\\\":\\\"string\\\"},{\\\"name\\\":\\\"field2\\\",\\\"type\\\":\\\"int\\\"}]}\",\n \"subject\" => \"test\",\n \"version\" => 13\n}}\n```","ref":"AvroSchema.html#module-performance"},{"type":"module","title":"Timestamps - AvroSchema","doc":"Avro timestamps are in Unix format with microsecond precision:\n\n```elixir\niex> datetime = DateTime.utc_now()\n~U[2019-11-08 09:09:01.055742Z]\n\niex> timestamp = AvroSchema.to_timestamp(datetime)\n1573204141055742\n\niex> datetime = AvroSchema.to_datetime(timestamp)\n~U[2019-11-08 09:09:01.055742Z]\n```\n\n# Contacts\n\nI am `jakemorrison` on on the Elixir Slack and Discord, `reachfh` on\nFreenode `#elixir-lang` IRC channel. Happy to chat or help with\nyour projects.","ref":"AvroSchema.html#module-timestamps"},{"type":"function","title":"AvroSchema.cache_registration/4","doc":"Cache registration.\n\nInserts the schema in the local cache.\n\n`register_schema/1` will then return the id without needing to communicate\nwith the Schema Registry.","ref":"AvroSchema.html#cache_registration/4"},{"type":"function","title":"AvroSchema.cache_schema/3","doc":"Cache schema locally.\n\nInserts the schema in the local cache under one or more references.\n\n`get_schema/1` will then return the schema without needing to communicate\nwith the Schema Registry.","ref":"AvroSchema.html#cache_schema/3"},{"type":"function","title":"Examples - AvroSchema.cache_schema/3","doc":"iex> schema_json = \"{\"name\":\"test\",\"type\":\"record\",\"fields\":[{\"name\":\"field1\",\"type\":\"string\"},{\"name\":\"field2\",\"type\":\"int\"}]}\"\n    iex> {:ok, schema} = AvroSchema.parse_schema(schema_json)\n    iex> full_name = AvroSchema.full_name(schema_json)\n    iex> fp = AvroSchema.fingerprint_schema(schema_json)\n    iex> ref = {full_name, fp}\n    iex> :ok = AvroSchema.cache_schema(ref, schema)\n    :ok","ref":"AvroSchema.html#cache_schema/3-examples"},{"type":"function","title":"AvroSchema.cache_schema_file/2","doc":"Cache schema files with fingerprints.\n\nLoads a schema file from disk, and parses it to get the full name.\n\nGenerates fingerprints for the schema using the raw file bytes,\n[Parsing Canonical Form](https://avro.apache.org/docs/1.8.2/spec.html#Parsing+Canonical+Form+for+Schemas),\nand normalized JSON for the canonical form (whitespace stripped).\nThis improves interop with fingerprints generated by other programs,\navoiding insignificant differences.\n\nAlso accepts a map with aliases that the schema should be registered\nunder, e.g. with a different subject name or legacy fingerprint.\nMap key is the full name, and value is a list of fingerprints or\n`{name, fingerprint}` tuples.\n\nDeduplicates the fingerprints and aliases, then calls `cache_schema/3`.","ref":"AvroSchema.html#cache_schema_file/2"},{"type":"function","title":"AvroSchema.canonicalize_schema/1","doc":"Ensure Avro schema is in Parsing Canonical Form.\n\nConverts schema into [Parsing Canonical Form](https://avro.apache.org/docs/current/spec.html#Parsing+Canonical+Form+for+Schemas)\nby decoding it and re-encoding it.","ref":"AvroSchema.html#canonicalize_schema/1"},{"type":"function","title":"AvroSchema.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AvroSchema.html#child_spec/1"},{"type":"function","title":"AvroSchema.create_fingerprint/1","doc":"Create CRC-64-AVRO fingerprint hash for Avro schema JSON.\n\nSee [CRC-64-AVRO](https://avro.apache.org/docs/1.8.2/spec.html#schema_fingerprints)","ref":"AvroSchema.html#create_fingerprint/1"},{"type":"function","title":"Examples - AvroSchema.create_fingerprint/1","doc":"iex> schema_json = \"{\"name\":\"test\",\"type\":\"record\",\"fields\":[{\"name\":\"field1\",\"type\":\"string\"},{\"name\":\"field2\",\"type\":\"int\"}]}\"\n    iex> AvroSchema.fingerprint_schema(schema_json)\n    <<172, 194, 58, 14, 16, 237, 158, 12>>","ref":"AvroSchema.html#create_fingerprint/1-examples"},{"type":"function","title":"AvroSchema.decode/2","doc":"Decode binary Avro data.","ref":"AvroSchema.html#decode/2"},{"type":"function","title":"AvroSchema.decode!/2","doc":"Decode binary Avro data, raises if there is a decoding error","ref":"AvroSchema.html#decode!/2"},{"type":"function","title":"AvroSchema.encode/2","doc":"Encode Avro data to binary.","ref":"AvroSchema.html#encode/2"},{"type":"function","title":"AvroSchema.encode!/2","doc":"Encode Avro data to binary, raises if there is an encoding error","ref":"AvroSchema.html#encode!/2"},{"type":"function","title":"AvroSchema.encode_schema/2","doc":"Encode parsed schema as JSON.","ref":"AvroSchema.html#encode_schema/2"},{"type":"function","title":"AvroSchema.fingerprint_schema/1","doc":"Create fingerprint of schema JSON.\n\nEnsures that schema is in standard form, then generates an\nCRC-64-AVRO fingerprint on it using `create_fingerprint/1`.","ref":"AvroSchema.html#fingerprint_schema/1"},{"type":"function","title":"Examples - AvroSchema.fingerprint_schema/1","doc":"iex> schema_json = \"{\"name\":\"test\",\"type\":\"record\",\"fields\":[{\"name\":\"field1\",\"type\":\"string\"},{\"name\":\"field2\",\"type\":\"int\"}]}\"\n    iex> AvroSchema.fingerprint_schema(schema_json)\n    <<172, 194, 58, 14, 16, 237, 158, 12>>","ref":"AvroSchema.html#fingerprint_schema/1-examples"},{"type":"function","title":"AvroSchema.full_name/1","doc":"Get full name field from schema.\n\n  This can be used as the Schema Registry subject.","ref":"AvroSchema.html#full_name/1"},{"type":"function","title":"AvroSchema.get_decoder/2","doc":"Get decoder function for registration id.\n\nConvenience function, calls `get_schema/1` on the id, then `make_decoder/2`.","ref":"AvroSchema.html#get_decoder/2"},{"type":"function","title":"AvroSchema.get_encoder/2","doc":"Get encoder function for registration id.\n\nConvenience function, calls `get_schema/1` on the id, then `make_encoder/2`.","ref":"AvroSchema.html#get_encoder/2"},{"type":"function","title":"AvroSchema.get_schema/1","doc":"Get schema for schema reference.\n\nThis tries to read the schema from the cache. If not found, it makes a call\nto the Schema Registry.\n\nThis is typically called by a Kafka consumer to find the schema which was\nused to encode data based on the tag.\n\nThis call has the overhead of an ETS lookup and potentially a GenServer call\nto fetch the Avro schema via HTTP. If you need maximum performance, keep the\nresult and reuse it for future requests with the same reference.","ref":"AvroSchema.html#get_schema/1"},{"type":"function","title":"AvroSchema.get_schema_files/2","doc":"List schema files in directory.\n\nLists files in a directory matching an extension, default `avsc`.","ref":"AvroSchema.html#get_schema_files/2"},{"type":"function","title":"AvroSchema.make_decoder/2","doc":"Make Avro decoder for schema.\n\nCreates a function which decodes a Avro encoded binary data to a map. By default,\na `:hook` option is provided that will convert all `:null` values to `nil`.","ref":"AvroSchema.html#make_decoder/2"},{"type":"function","title":"AvroSchema.make_encoder/2","doc":"Make Avro encoder for schema.\n\nCreates a function which encodes Avro terms to binary.","ref":"AvroSchema.html#make_encoder/2"},{"type":"function","title":"AvroSchema.make_fp_subject/1","doc":"Make registration subject from name + fingerprint.","ref":"AvroSchema.html#make_fp_subject/1"},{"type":"function","title":"AvroSchema.make_fp_subject/2","doc":"","ref":"AvroSchema.html#make_fp_subject/2"},{"type":"function","title":"AvroSchema.normalize_json/1","doc":"Normalize JSON by decoding and re-encoding it.\n\nThis reduces irrelevant differences such as whitespace which may affect\nfingerprinting.","ref":"AvroSchema.html#normalize_json/1"},{"type":"function","title":"AvroSchema.parse_schema/1","doc":"Parse schema into Avro library internal form.","ref":"AvroSchema.html#parse_schema/1"},{"type":"function","title":"Examples - AvroSchema.parse_schema/1","doc":"iex> schema_json = \"{\"name\":\"test\",\"type\":\"record\",\"fields\":[{\"name\":\"field1\",\"type\":\"string\"},{\"name\":\"field2\",\"type\":\"int\"}]}\"\n  iex> {:ok, schema} = AvroSchema.parse_schema(schema_json)\n  {:ok,\n   {:avro_record_type, \"test\", \"\", \"\", [],\n    [\n      {:avro_record_field, \"field1\", \"\", {:avro_primitive_type, \"string\", []},\n        :undefined, :ascending, []},","ref":"AvroSchema.html#parse_schema/1-examples"},{"type":"function","title":"AvroSchema.register_schema/2","doc":"Register schema in Confluent Schema Registry.\n\nThe subject is a unique name to register the schema, often the full name from the Avro schema.\nSee the [standard strategies](https://docs.confluent.io/current/schema-registry/serializer-formatter.html#subject-name-strategy)\nused by Confluent in their Kafka libraries.\n\nIt is safe to register the same schema multiple times, it will always return\nthe same ID.","ref":"AvroSchema.html#register_schema/2"},{"type":"function","title":"AvroSchema.start_link/2","doc":"Start cache GenServer.","ref":"AvroSchema.html#start_link/2"},{"type":"function","title":"AvroSchema.stop/0","doc":"Stop cache GenServer","ref":"AvroSchema.html#stop/0"},{"type":"function","title":"AvroSchema.tag/2","doc":"Tag Avro binary data with schema that created it.\n\nAdds a tag to the front of data indicating the schema that was used\nto encode it.\n\nUses\n[Confluent wire format](https://docs.confluent.io/current/schema-registry/serializer-formatter.html#wire-format)\nfor integer registry IDs and\n[Avro single object encoding](https://avro.apache.org/docs/1.8.2/spec.html#single_object_encoding_spec)\nfor fingerprints.\n\nThis function matches schema IDs as integers and encodes them using Confluent\nformat, and fingerprints as binary and encodes them as Avro.\n\nStrictly speaking, however, fingerprints are integers, so make sure that you\nconvert them to binary before calling this function.\n\nNote that this function returns an iolist for efficiency, not a binary.","ref":"AvroSchema.html#tag/2"},{"type":"function","title":"Examples - AvroSchema.tag/2","doc":"iex> schema_json = \"{\"name\":\"test\",\"type\":\"record\",\"fields\":[{\"name\":\"field1\",\"type\":\"string\"},{\"name\":\"field2\",\"type\":\"int\"}]}\"\n    iex> fp = AvroSchema.fingerprint_schema(schema_json)\n    iex> AvroSchema.tag(\"hello\", fp)\n    [<<195, 1>>, <<172, 194, 58, 14, 16, 237, 158, 12>>, \"hello\"]","ref":"AvroSchema.html#tag/2-examples"},{"type":"function","title":"AvroSchema.to_datetime/1","doc":"Convert Avro integer timestamp to `DateTime`.\n\n    iex> timestamp = 1573204141055742\n    iex> datetime = AvroSchema.to_datetime(timestamp)\n    ~U[2019-11-08 09:09:01.055742Z]","ref":"AvroSchema.html#to_datetime/1"},{"type":"function","title":"AvroSchema.to_hex/1","doc":"Convert binary fingerprint to hex","ref":"AvroSchema.html#to_hex/1"},{"type":"function","title":"AvroSchema.to_timestamp/1","doc":"Convert `DateTime` to Avro integer timestamp with ms precision.","ref":"AvroSchema.html#to_timestamp/1"},{"type":"function","title":"Examples - AvroSchema.to_timestamp/1","doc":"iex> datetime = DateTime.utc_now()\n    ~U[2019-11-08 09:09:01.055742Z]\n\n    iex> timestamp = AvroSchema.to_timestamp(datetime)\n    1573204141055742","ref":"AvroSchema.html#to_timestamp/1-examples"},{"type":"function","title":"AvroSchema.untag/1","doc":"Split schema tag from tagged Avro binary data.\n\nSupports [Confluent wire format](https://docs.confluent.io/current/schema-registry/serializer-formatter.html#wire-format)\nfor integer registry IDs and [Avro single object encoding](https://avro.apache.org/docs/1.8.2/spec.html#single_object_encoding_spec)\nfor fingerprints.","ref":"AvroSchema.html#untag/1"},{"type":"function","title":"Examples - AvroSchema.untag/1","doc":"iex> tagged_avro = IO.iodata_to_binary([<<195, 1>>, <<172, 194, 58, 14, 16, 237, 158, 12>>, \"hello\"])\n  iex> AvroSchema.untag(tagged_avro)\n  {:ok, {{:avro, <<172, 194, 58, 14, 16, 237, 158, 12>>}, \"hello\"}}\n\n  iex> tagged_confluent = IO.iodata_to_binary([<<0, 0, 0, 0, 7>>, \"hello\"])\n  iex> AvroSchema.untag(tagged_confluent)\n  {:ok, {{:confluent, 7}, \"hello\"}}","ref":"AvroSchema.html#untag/1-examples"},{"type":"type","title":"AvroSchema.cache_value/0","doc":"Cache value","ref":"AvroSchema.html#t:cache_value/0"},{"type":"type","title":"AvroSchema.client/0","doc":"Tesla client","ref":"AvroSchema.html#t:client/0"},{"type":"type","title":"AvroSchema.decoded/0","doc":"","ref":"AvroSchema.html#t:decoded/0"},{"type":"type","title":"AvroSchema.fp/0","doc":"Fingerprint, normally CRC-64-AVRO but could be e.g. MD5","ref":"AvroSchema.html#t:fp/0"},{"type":"type","title":"AvroSchema.ref/0","doc":"Cache key","ref":"AvroSchema.html#t:ref/0"},{"type":"type","title":"AvroSchema.regid/0","doc":"Integer ID returned by Schema Registry","ref":"AvroSchema.html#t:regid/0"},{"type":"type","title":"AvroSchema.subject/0","doc":"Subject in Schema Registry / Avro name","ref":"AvroSchema.html#t:subject/0"},{"type":"extras","title":"AvroSchema","doc":"# AvroSchema\n\n---\n\n[![CircleCI](https://circleci.com/gh/cogini/avro_schema.svg?style=svg)](https://app.circleci.com/github/cogini/avro_schema/pipelines)\n\nThis is a library for working with [Avro](https://avro.apache.org/)\nschemas and the [Confluent® Schema Registry](https://www.confluent.io/confluent-schema-registry),\nprimarily focused on working with [Kafka](https://kafka.apache.org/) streams.\nIt relies on [erlavro](https://github.com/klarna/erlavro) for encoding and\ndecoding data and [confluent_schema_registry](https://github.com/cogini/confluent_schema_registry)\nto look up schemas using the [Schema Registry API](https://docs.confluent.io/current/schema-registry/develop/api.html).\n\nIts primary value is that it caches schemas for performance and to allow\nprograms to work independently of the Schema Registry being available.\nIt also has a consistent set of functions to manage schema tags, look up\nschemas from the Schema Registry or files, and encode/decode data.\n\nMuch thanks to Klarna for [Avlizer](https://github.com/klarna/avlizer), which\nprovides similar functionality to this library in Erlang,\n[erlavro](https://github.com/klarna/erlavro) for Avro, and\n[brod](https://github.com/klarna/brod) for dealing with Kafka.","ref":"readme.html"},{"type":"extras","title":"Installation - AvroSchema","doc":"Add the package to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:avro_schema, \"~> 0.1.0\"}]\nend\n```\n\nThen run `mix deps.get` to fetch the new dependency.\n\nDocumentation is on [HexDocs](https://hexdocs.pm/avro_schema).\nTo generate a local copy, run `mix docs`.","ref":"readme.html#installation"},{"type":"extras","title":"Starting - AvroSchema","doc":"Add the cache GenServer to your application's supervision tree:\n\n```elixir\ndef start(_type, _args) do\n  cache_dir = Application.get_env(:yourapp, :cache_dir, \"/tmp\")\n\n  children = [\n    {AvroSchema, [cache_dir: cache_dir]},\n  ]\n\n  opts = [strategy: :one_for_one, name: LogElasticsearch.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"readme.html#starting"},{"type":"extras","title":"Overview - AvroSchema","doc":"When using Kafka, producers and consumers are separated, and schemas may evolve\nover time. It is common for producers to tag data indicating the schema that\nwas used to encode it. Consumers can then look up the corresponding schema\nversion and use it decode the data.\n\nThis library supports two tagging formats, [Confluent wire format](https://docs.confluent.io/current/schema-registry/serializer-formatter.html#wire-format),\nand [Avro single object encoding](https://avro.apache.org/docs/1.8.2/spec.html#single_object_encoding_spec).","ref":"readme.html#overview"},{"type":"extras","title":"Confluent wire format - AvroSchema","doc":"With the [Confluent wire format](https://docs.confluent.io/current/schema-registry/serializer-formatter.html#wire-format),\nAvro binary encoded objects are prefixed with a five-byte tag.\n\nThe first byte indicates the Confluent serialization format version number,\ncurrently always 0. The following four bytes encode the integer schema ID as\nreturned from the Schema Registry in network byte order.","ref":"readme.html#confluent-wire-format"},{"type":"extras","title":"Avro single-object encoding - AvroSchema","doc":"When used without a schema registry, it's common to prefix binary data with a\nhash of the schema that created it. In the past, that might be something like\nMD5.\n\nThe Avro \"Single-object encoding\" formalizes this, prefixing Avro binary data\nwith a two-byte marker, C3 01, to show that the message is Avro and uses this\nsingle-record format (version 1). That is followed by the 8-byte little-endian\n[CRC-64-AVRO](https://avro.apache.org/docs/1.8.2/spec.html#schema_fingerprints)\nfingerprint of the object's schema.\n\nThe CRC64 algorithm is uncommon, but used because it is relatively short,\nwhile still being good enough to detect collisions. The fingerprint function is\nimplemented in `fingerprint_schema/1`.","ref":"readme.html#avro-single-object-encoding"},{"type":"extras","title":"Schema Registry - AvroSchema","doc":"In a relatively static system, it's not too hard to exchange schema files\nbetween producers and consumers. When things are changing more frequently, it\ncan be difficult to keep files up to date. It's also easy for insignificant\ndifferences such as whitespace to result in different schema hashes.\n\nThe Schema Registry solves this by providing a centralized service which\nproducers and consumers can call to get a unique identifier for a schema\nversion. Producers register a schema with the service and get an id.\nConsumers look up the id to get the schema.\n\nThe Schema Registry also does validation on new schemas to ensure that they\nmeet a backwards compatibility policy for the organization.\nThis helps you to [evolve schemas](https://docs.confluent.io/current/schema-registry/avro.html)\nover time and deploy them without breaking running applications.\n\nThe disadvantage of the Schema Registry is that it can be a single point\nof failure. Different schema registries will in general assign a different\nnumeric id to the same schema.\n\nThis library provides functions to register schemas with the Schema Registry\nand look them up by id. It caches the results in RAM (ETS) for performance,\nand optionally also on disk (DETS). This gives good performance and allows\nprograms to work without needing to communicate with the Schema Registry.\nOnce read, the numeric IDs never change, so it's safe to cache them indefinitely.\n\nThe library also has support for managing schemas from files. It can add files\nto the cache by fingerprint, registering the same schema under multiple\nfingerprints, i.e. the raw JSON, a version in\n[Parsing Canonical Form](https://avro.apache.org/docs/current/spec.html#Parsing+Canonical+Form+for+Schemas)\nand with whitespace stripped out. You can also manually register aliases for the\nname and fingerprint to handle legacy data.","ref":"readme.html#schema-registry"},{"type":"extras","title":"Kafka producer example - AvroSchema","doc":"A Kafka producer program needs to be able to encode the data with an Avro\nschema and tag it with the schema ID or fingerprint. It may store the\nschema in the code or read it from a file, or it may look it up from the Schema\nRegistry using the subject.\n\nThe subject is a registered name which identifies the type of data.\nThere are are several [standard strategies](https://docs.confluent.io/current/schema-registry/serializer-formatter.html#subject-name-strategy)\nused by Confluent in their Kafka libraries.\n\n* `TopicNameStrategy`, the default, registers the schema based on the Kafka\n  topic name, implicitly requiring that all messages use the same schema.\n\n* `RecordNameStrategy` names the schema using the record type, allowing\n  a single topic to have multiple different types of data or multiple topics\n  to have the same type of data.\n\n  In an Avro schema the \"[full name](https://avro.apache.org/docs/1.8.2/spec.html#names)\", is\n  a namespace-qualified name for the record, e.g. `com.example.X`. In the schema, it is\n  the `name` field.\n\n* `TopicRecordNameStrategy` names the schema using a combination of topic and record.\n\nWith the subject, the producer can call the Schema Registry to get the ID\nmatching the Avro schema:\n\n```elixir\niex> schema_json = \"{\\\"name\\\":\\\"test\\\",\\\"type\\\":\\\"record\\\",\\\"fields\\\":[{\\\"name\\\":\\\"field1\\\",\\\"type\\\":\\\"string\\\"},{\\\"name\\\":\\\"field2\\\",\\\"type\\\":\\\"int\\\"}]}\"\niex> subject = \"test\"\niex> {:ok, ref} = AvroSchema.register_schema(subject, schema_json)\n{:ok, 21}\n```\n\nIf the schema has already been registered, then the Schema Registry will\nreturn the current id. If you are registering a new version of the schema, then\nthe Schema Registry will first check if it is compatible with the old one.\nDepending on the compatibility rules, it may reject the schema.\n\nThe producer next needs to get an encoder for the schema.\n\nThe encoder is a function that takes Avro key/value data and encodes\nit to a binary.\n\n```elixir\niex> {:ok, encoder} = AvroSchema.make_encoder(schema_json)\n{:ok, #Function<2.110795165/1 in :avro.make_simple_encoder/2>}\n```\n\nNext we encode some data:\n\n```elixir\niex> data = %{field1: \"hello\", field2: 100}\niex> encoded = AvroSchema.encode!(data, encoder)\n[['\\n', \"hello\"], [200, 1]]\n```\n\nFinally, we tag the data:\n\n```elixir\niex> tagged_confluent = AvroSchema.tag(encoded, 21)\n[<<0, 0, 0, 0, 21>>, [['\\n', \"hello\"], [200, 1]]]\n```\n\nIf you are using files, the process is similar. First\ncreate a fingerprint for the schema:\n\n```elixir\niex> fp = AvroSchema.fingerprint_schema(schema_json)\n<<172, 194, 58, 14, 16, 237, 158, 12>>\n```\n\nNext tag the data:\n\n```elixir\niex> tagged_avro = AvroSchema.tag(encoded, fp)\n[\n  <<195, 1>>,\n  <<172, 194, 58, 14, 16, 237, 158, 12>>,\n  [['\\n', \"hello\"], [200, 1]]\n]\n```\n\nNow you can send the data to Kafka.","ref":"readme.html#kafka-producer-example"},{"type":"extras","title":"Kafka consumer example - AvroSchema","doc":"The process for a consumer is similar.\n\nReceive the data and get the registration id in Confluent format:\n\n```elixir\niex> tagged_confluent = IO.iodata_to_binary(AvroSchema.tag(encoded, 21))\n<<0, 0, 0, 0, 21, 10, 104, 101, 108, 108, 111, 200, 1>>\n\niex> {:ok, {{:confluent, regid}, bin}} = AvroSchema.untag(tagged_confluent)\n{:ok, {{:confluent, 21}, <<10, 104, 101, 108, 108, 111, 200, 1>>}}\n```\n\nGet the schema from the Schema Registry:\n\n```elixir\niex> {:ok, schema} = AvroSchema.get_schema(regid)\n{:ok,\n {:avro_record_type, \"test\", \"\", \"\", [],\n  [\n    {:avro_record_field, \"field1\", \"\", {:avro_primitive_type, \"string\", []},\n     :undefined, :ascending, []},\n    {:avro_record_field, \"field2\", \"\", {:avro_primitive_type, \"int\", []},\n     :undefined, :ascending, []}\n  ], \"test\", []}}\n```\n\nCreate a decoder and decode the data:\n\n```elixir\niex> {:ok, decoder} = AvroSchema.make_decoder(schema)\n{:ok, #Function<4.110795165/1 in :avro.make_simple_decoder/2>}\n\niex> decoded = AvroSchema.decode!(bin, decoder)\n%{\"field1\" => \"hello\", \"field2\" => 100}\n```\n\nThe process is similar with a fingerprint.  In this case, we get the schema\nfrom files and register it in the cache using the schema name and fingerprints. There\nis more than one fingerprint because we register it with the raw schema from\nthe file and the normalized JSON for better interop.\n\n```elixir\niex> {:ok, files} = AvroSchema.get_schema_files(\"test/schemas\")\n{:ok, [\"test/schemas/test.avsc\"]}\n\niex> for file <- files, do: AvroSchema.cache_schema_file(file)\n[\n  ok: [\n    {\"test\", <<172, 194, 58, 14, 16, 237, 158, 12>>},\n    {\"test\", <<194, 132, 80, 199, 36, 146, 103, 147>>}\n  ]\n]\n```\n\nTo decode, separate the fingerprint from the data:\n\n```elixir\niex> tagged_avro = IO.iodata_to_binary(AvroSchema.tag(encoded, fp))\n<<195, 1, 172, 194, 58, 14, 16, 237, 158, 12, 10, 104, 101, 108, 108, 111, 200,\n  1>>\n\niex> {:ok, {{:avro, fp}, bin}} = AvroSchema.untag(tagged_avro)\n{:ok, {{:avro, <<172, 194, 58, 14, 16, 237, 158, 12>>},\n  <<10, 104, 101, 108, 108, 111, 200, 1>>}}\n```\n\nGet the decoder and decode the data:\n\n```elixir\niex> {:ok, schema} = AvroSchema.get_schema({\"test\", fp})\n{:ok,\n {:avro_record_type, \"test\", \"\", \"\", [],\n  [\n    {:avro_record_field, \"field1\", \"\", {:avro_primitive_type, \"string\", []},\n     :undefined, :ascending, []},\n    {:avro_record_field, \"field2\", \"\", {:avro_primitive_type, \"int\", []},\n     :undefined, :ascending, []}\n  ], \"test\", []}}\n```\n\nDecoding works the same as with the Schema Registry:\n\n```elixir\niex> {:ok, decoder} = AvroSchema.make_decoder(schema)\n{:ok, #Function<4.110795165/1 in :avro.make_simple_decoder/2>}\n\niex> decoded = AvroSchema.decode!(bin, decoder)\n%{\"field1\" => \"hello\", \"field2\" => 100}\n```","ref":"readme.html#kafka-consumer-example"},{"type":"extras","title":"Performance - AvroSchema","doc":"For best performance, save the encoder or decoder in your process\nstate to avoid the overhead of looking it up for each message.\n\nAn in-memory ETS cache maps the integer registry ID or name + fingerprint\nto the corresponding schema and decoder. It also allows lookups using name and\nfingerprint as a key.\n\nThe fingerprint is CRC64 by default. You can also register a name with your own\nfingerprint.\n\nThis library also allows consumers to look up the schema on demand from the\nSchema Registry using the name + fingerprint as the registry subject name.\n\nThis library can optionally persist the cache data on disk using DETS,\nallowing programs to work without continuous access to the Schema Registry.\n\nPrograms which use Kafka may process high message volumes, so efficiency\nis important. They generally use multiple processes, typically one per\ntopic partition or more. On startup, each process may simultaneously attempt to\nlook up schemas.\n\nThe cache lookup runs in the caller's process, so it can run in parallel.\nIf there is a cache miss, then it calls the GenServer to update the cache.\nThis has the effect of serializing requests, ensuring that only one runs\nat a time. See https://www.cogini.com/blog/avoiding-genserver-bottlenecks/ for\ndiscussion.\n\nIn order to improve interoperability, the schema should be put into standard form.\n\nIt might also call the schema registry to get the schema for a given subject:\n\n```elixir\niex> ConfluentSchemaRegistry.get_schema(client, \"test\")\n{:ok,\n%{\n \"id\" => 21,\n \"schema\" => \"{\\\"type\\\":\\\"record\\\",\\\"name\\\":\\\"test\\\",\\\"fields\\\":[{\\\"name\\\":\\\"field1\\\",\\\"type\\\":\\\"string\\\"},{\\\"name\\\":\\\"field2\\\",\\\"type\\\":\\\"int\\\"}]}\",\n \"subject\" => \"test\",\n \"version\" => 13\n}}\n```","ref":"readme.html#performance"},{"type":"extras","title":"Timestamps - AvroSchema","doc":"Avro timestamps are in Unix format with microsecond precision:\n\n```elixir\niex> datetime = DateTime.utc_now()\n~U[2019-11-08 09:09:01.055742Z]\n\niex> timestamp = AvroSchema.to_timestamp(datetime)\n1573204141055742\n\niex> datetime = AvroSchema.to_datetime(timestamp)\n~U[2019-11-08 09:09:01.055742Z]\n```\n\n# Contacts\n\nI am `jakemorrison` on on the Elixir Slack and Discord, `reachfh` on\nFreenode `#elixir-lang` IRC channel. Happy to chat or help with\nyour projects.","ref":"readme.html#timestamps"},{"type":"extras","title":"Changelog","doc":"# Changelog\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.4.0] - 2024-04-17","ref":"changelog.html"},{"type":"extras","title":"Changed - Changelog","doc":"- Update libs\n\n## [0.3.0] - 2020-02-17","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Pull in [erlavro 2.9.0](https://github.com/klarna/erlavro/compare/e43fe8b..cd49689) which encodes atom values as strings for string types\n\n## [0.2.0] - 2020-02-05","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Make `encode/2` return `{:ok, binary} | {:error, term()}` and add new `encode!/2`,\n  https://github.com/cogini/avro_schema/pull/6\n- Make `decode/2` return `{:ok, binary} | {:error, term()}` and add new `decode!/2`,\n  https://github.com/cogini/avro_schema/pull/14\n- Make `make_decoder/2` return maps as map type by default\n  https://github.com/cogini/avro_schema/pull/13\n- Make `make_decoder/2` decodes \"null\" schema types as `nil` by default by\n  providing a decoder hook\n  https://github.com/cogini/avro_schema/pull/16\n- Encode nil values correctly for \"null\" schema types,\n  https://github.com/cogini/avro_schema/pull/8\n- Make `do_register_schema/3` private\n- Rename `make_subject/1 and make_subject/2 to make_fp_subject/1 and make_fp_subject/2` to make name less generic\n\n## [0.1.0] - 2019-12-31","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release","ref":"changelog.html#added"},{"type":"extras","title":"License (Apache-2.0)","doc":"# Apache License\nVersion 2.0, January 2004\n\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION","ref":"license.html"},{"type":"extras","title":"1. Definitions. - License (Apache-2.0)","doc":"\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.","ref":"license.html#1-definitions"},{"type":"extras","title":"2. Grant of Copyright License. - License (Apache-2.0)","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.","ref":"license.html#2-grant-of-copyright-license"},{"type":"extras","title":"3. Grant of Patent License. - License (Apache-2.0)","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work or a\nContribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.","ref":"license.html#3-grant-of-patent-license"},{"type":"extras","title":"4. Redistribution. - License (Apache-2.0)","doc":"You may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n1. You must give any other recipients of the Work or Derivative Works a copy of\n   this License; and\n\n2. You must cause any modified files to carry prominent notices stating that\n   You changed the files; and\n\n3. You must retain, in the Source form of any Derivative Works that You\n   distribute, all copyright, patent, trademark, and attribution notices from\n   the Source form of the Work, excluding those notices that do not pertain to\n   any part of the Derivative Works; and\n\n4. If the Work includes a \"NOTICE\" text file as part of its distribution, then\n   any Derivative Works that You distribute must include a readable copy of the\n   attribution notices contained within such NOTICE file, excluding those\n   notices that do not pertain to any part of the Derivative Works, in at least\n   one of the following places: within a NOTICE text file distributed as part\n   of the Derivative Works; within the Source form or documentation, if\n   provided along with the Derivative Works; or, within a display generated by\n   the Derivative Works, if and wherever such third-party notices normally\n   appear. The contents of the NOTICE file are for informational purposes only\n   and do not modify the License. You may add Your own attribution notices\n   within Derivative Works that You distribute, alongside or as an addendum to\n   the NOTICE text from the Work, provided that such additional attribution\n   notices cannot be construed as modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.","ref":"license.html#4-redistribution"},{"type":"extras","title":"5. Submission of Contributions. - License (Apache-2.0)","doc":"Unless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.","ref":"license.html#5-submission-of-contributions"},{"type":"extras","title":"6. Trademarks. - License (Apache-2.0)","doc":"This License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.","ref":"license.html#6-trademarks"},{"type":"extras","title":"7. Disclaimer of Warranty. - License (Apache-2.0)","doc":"Unless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE, NON-\nINFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.","ref":"license.html#7-disclaimer-of-warranty"},{"type":"extras","title":"8. Limitation of Liability. - License (Apache-2.0)","doc":"In no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.","ref":"license.html#8-limitation-of-liability"},{"type":"extras","title":"9. Accepting Warranty or Additional Liability. - License (Apache-2.0)","doc":"While redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS\n\nCopyright 2021-2023, Jake Morrison  .\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\nhttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"license.html#9-accepting-warranty-or-additional-liability"},{"type":"extras","title":"Contributing","doc":"# Contributing to Uinta\n\nPlease take a moment to review this document in order to make the contribution\nprocess easy and effective for everyone involved!\nAlso make sure you read our [Code of Conduct](CODE_OF_CONDUCT.md) that outlines our commitment towards an open and welcoming environment.","ref":"contributing.html"},{"type":"extras","title":"Using the issue tracker - Contributing","doc":"Use the issues tracker for:\n\n* [Bug reports](#bug-reports)\n* [Submitting pull requests](#pull-requests)\n\nWe do our best to keep the issue tracker tidy and organized, making it useful\nfor everyone. For example, we classify open issues per perceived difficulty,\nmaking it easier for developers to [contribute to Uinta](#pull-requests).","ref":"contributing.html#using-the-issue-tracker"},{"type":"extras","title":"Bug reports - Contributing","doc":"A bug is either a _demonstrable problem_ that is caused by the code in the repository,\nor indicate missing, unclear, or misleading documentation. Good bug reports are extremely\nhelpful - thank you!\n\nGuidelines for bug reports:\n\n1. **Use the GitHub issue search** &mdash; check if the issue has already been\n   reported.\n\n2. **Check if the issue has been fixed** &mdash; try to reproduce it using the\n   `master` branch in the repository.\n\n3. **Isolate and report the problem** &mdash; ideally create a reduced test\n   case.\n\nPlease try to be as detailed as possible in your report. Include information about\nyour Operating System, as well as your Erlang, Elixir and Uinta versions. Please provide steps to\nreproduce the issue as well as the outcome you were expecting! All these details\nwill help developers to fix any potential bugs.\n\nExample:\n\n> Short and descriptive example bug report title\n>\n> A summary of the issue and the environment in which it occurs. If suitable,\n> include the steps required to reproduce the bug.\n>\n> 1. This is the first step\n> 2. This is the second step\n> 3. Further steps, etc.\n>\n> ` ` - a link to the reduced test case (e.g. a GitHub Gist)\n>\n> Any other information you want to share that is relevant to the issue being\n> reported. This might include the lines of code that you have identified as\n> causing the bug, and potential solutions (and your opinions on their\n> merits).","ref":"contributing.html#bug-reports"},{"type":"extras","title":"Contributing Documentation - Contributing","doc":"Code documentation (`@doc`, `@moduledoc`, `@typedoc`) has a special convention:\nthe first paragraph is considered to be a short summary.\n\nFor functions, macros and callbacks say what it will do. For example write\nsomething like:\n\n```elixir\n@doc \"\"\"\nMarks the given value as HTML safe.\n\"\"\"\ndef safe({:safe, value}), do: {:safe, value}\n```\n\nFor modules, protocols and types say what it is. For example write\nsomething like:\n\n```elixir\ndefmodule MyModule do\n  @moduledoc \"\"\"\n  Conveniences for working HTML strings and templates.\n  ...\n  \"\"\"\n```\n\nKeep in mind that the first paragraph might show up in a summary somewhere, long\ntexts in the first paragraph create very ugly summaries. As a rule of thumb\nanything longer than 80 characters is too long.\n\nTry to keep unnecessary details out of the first paragraph, it's only there to\ngive a user a quick idea of what the documented \"thing\" does/is. The rest of the\ndocumentation string can contain the details, for example when a value and when\n`nil` is returned.\n\nIf possible include examples, preferably in a form that works with doctests.\nThis makes it easy to test the examples so that they don't go stale and examples\nare often a great help in explaining what a function does.","ref":"contributing.html#contributing-documentation"},{"type":"extras","title":"Pull requests - Contributing","doc":"Good pull requests - patches, improvements, new features - are a fantastic\nhelp. They should remain focused in scope and avoid containing unrelated\ncommits.\n\n**IMPORTANT**: By submitting a patch, you agree that your work will be\nlicensed under the license used by the project.\n\nIf you have any large pull request in mind (e.g. implementing features,\nrefactoring code, etc), **please ask first** otherwise you risk spending\na lot of time working on something that the project's developers might\nnot want to merge into the project.\n\nPlease adhere to the coding conventions in the project (indentation,\naccurate comments, etc.) and don't forget to add your own tests and\ndocumentation. When working with git, we recommend the following process\nin order to craft an excellent pull request:\n\n1. [Fork](https://help.github.com/articles/fork-a-repo/) the project, clone your fork,\n   and configure the remotes:\n\n   ```bash\n   # Clone your fork of the repo into the current directory\n   git clone https://github.com/ /uinta\n\n   # Navigate to the newly cloned directory\n   cd uinta\n\n   # Assign the original repo to a remote called \"upstream\"\n   git remote add upstream https://github.com/podium/uinta\n   ```\n\n2. If you cloned a while ago, get the latest changes from upstream, and update your fork:\n\n   ```bash\n   git checkout master\n   git pull upstream master\n   git push\n   ```\n\n3. Create a new topic branch (off of `master`) to contain your feature, change,\n   or fix.\n\n   **IMPORTANT**: Making changes in `master` is discouraged. You should always\n   keep your local `master` in sync with upstream `master` and make your\n   changes in topic branches.\n\n   ```bash\n   git checkout -b  \n   ```\n\n4. Commit your changes in logical chunks. Keep your commit messages organized,\n   with a short description in the first line and more detailed information on\n   the following lines. Feel free to use Git's\n   [interactive rebase](https://help.github.com/articles/about-git-rebase/)\n   feature to tidy up your commits before making them public.\n\n5. Make sure all the tests are still passing.\n\n   ```bash\n   mix test\n   ```\n\n6. Push your topic branch up to your fork:\n\n   ```bash\n   git push origin  \n   ```\n\n7. [Open a Pull Request](https://help.github.com/articles/about-pull-requests/)\n    with a clear title and description.\n\n8. If you haven't updated your pull request for a while, you should consider\n   rebasing on master and resolving any conflicts.\n\n   **IMPORTANT**: _Never ever_ merge upstream `master` into your branches. You\n   should always `git rebase` on `master` to bring your changes up to date when\n   necessary.\n\n   ```bash\n   git checkout master\n   git pull upstream master\n   git checkout  \n   git rebase master\n   ```\n\nThank you for your contributions!","ref":"contributing.html#pull-requests"},{"type":"extras","title":"Guides - Contributing","doc":"These Guides aim to be inclusive. We use \"we\" and \"our\" instead of \"you\" and\n\"your\" to foster this sense of inclusion.\n\nIdeally there is something for everybody in each guide, from beginner to expert.\nThis is hard, maybe impossible. When we need to compromise, we do so on behalf\nof beginning users because expert users have more tools at their disposal to\nhelp themselves.\n\nThe general pattern we use for presenting information is to first introduce a\nsmall, discrete topic, then write a small amount of code to demonstrate the\nconcept, then verify that the code worked.\n\nIn this way, we build from small, easily digestible concepts into more complex\nones. The shorter this cycle is, as long as the information is still clear and\ncomplete, the better.\n\nFor formatting the guides:\n\n- We use the `elixir` code fence for all module code.\n- We use the `iex` for IEx sessions.\n- We use the `console` code fence for shell commands.\n- We use the `html` code fence for html templates, even if there is elixir code\n  in the template.\n- We use backticks for filenames and directory paths.\n- We use backticks for module names, function names, and variable names.\n- Documentation line length should hard wrapped at around 100 characters if possible.","ref":"contributing.html#guides"},{"type":"extras","title":"Code of Conduct","doc":"# Contributor Covenant Code of Conduct","ref":"code_of_conduct.html"},{"type":"extras","title":"Our Pledge - Code of Conduct","doc":"We as members, contributors, and leaders pledge to make participation in our\ncommunity a harassment-free experience for everyone, regardless of age, body\nsize, visible or invisible disability, ethnicity, sex characteristics, gender\nidentity and expression, level of experience, education, socio-economic status,\nnationality, personal appearance, race, caste, color, religion, or sexual\nidentity and orientation.\n\nWe pledge to act and interact in ways that contribute to an open, welcoming,\ndiverse, inclusive, and healthy community.","ref":"code_of_conduct.html#our-pledge"},{"type":"extras","title":"Our Standards - Code of Conduct","doc":"Examples of behavior that contributes to a positive environment for our\ncommunity include:\n\n* Demonstrating empathy and kindness toward other people\n* Being respectful of differing opinions, viewpoints, and experiences\n* Giving and gracefully accepting constructive feedback\n* Accepting responsibility and apologizing to those affected by our mistakes,\n  and learning from the experience\n* Focusing on what is best not just for us as individuals, but for the overall\n  community\n\nExamples of unacceptable behavior include:\n\n* The use of sexualized language or imagery, and sexual attention or advances of\n  any kind\n* Trolling, insulting or derogatory comments, and personal or political attacks\n* Public or private harassment\n* Publishing others' private information, such as a physical or email address,\n  without their explicit permission\n* Other conduct which could reasonably be considered inappropriate in a\n  professional setting","ref":"code_of_conduct.html#our-standards"},{"type":"extras","title":"Enforcement Responsibilities - Code of Conduct","doc":"Community leaders are responsible for clarifying and enforcing our standards of\nacceptable behavior and will take appropriate and fair corrective action in\nresponse to any behavior that they deem inappropriate, threatening, offensive,\nor harmful.\n\nCommunity leaders have the right and responsibility to remove, edit, or reject\ncomments, commits, code, wiki edits, issues, and other contributions that are\nnot aligned to this Code of Conduct, and will communicate reasons for moderation\ndecisions when appropriate.","ref":"code_of_conduct.html#enforcement-responsibilities"},{"type":"extras","title":"Scope - Code of Conduct","doc":"This Code of Conduct applies within all community spaces, and also applies when\nan individual is officially representing the community in public spaces.\nExamples of representing our community include using an official e-mail address,\nposting via an official social media account, or acting as an appointed\nrepresentative at an online or offline event.","ref":"code_of_conduct.html#scope"},{"type":"extras","title":"Enforcement - Code of Conduct","doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be\nreported to the community leaders responsible for enforcement at\nengineering@therealreal.com. All complaints will be reviewed and investigated\npromptly and fairly.\n\nAll community leaders are obligated to respect the privacy and security of the\nreporter of any incident.","ref":"code_of_conduct.html#enforcement"},{"type":"extras","title":"Enforcement Guidelines - Code of Conduct","doc":"Community leaders will follow these Community Impact Guidelines in determining\nthe consequences for any action they deem in violation of this Code of Conduct:","ref":"code_of_conduct.html#enforcement-guidelines"},{"type":"extras","title":"1. Correction - Code of Conduct","doc":"**Community Impact**: Use of inappropriate language or other behavior deemed\nunprofessional or unwelcome in the community.\n\n**Consequence**: A private, written warning from community leaders, providing\nclarity around the nature of the violation and an explanation of why the\nbehavior was inappropriate. A public apology may be requested.","ref":"code_of_conduct.html#1-correction"},{"type":"extras","title":"2. Warning - Code of Conduct","doc":"**Community Impact**: A violation through a single incident or series of\nactions.\n\n**Consequence**: A warning with consequences for continued behavior. No\ninteraction with the people involved, including unsolicited interaction with\nthose enforcing the Code of Conduct, for a specified period of time. This\nincludes avoiding interactions in community spaces as well as external channels\nlike social media. Violating these terms may lead to a temporary or permanent\nban.","ref":"code_of_conduct.html#2-warning"},{"type":"extras","title":"3. Temporary Ban - Code of Conduct","doc":"**Community Impact**: A serious violation of community standards, including\nsustained inappropriate behavior.\n\n**Consequence**: A temporary ban from any sort of interaction or public\ncommunication with the community for a specified period of time. No public or\nprivate interaction with the people involved, including unsolicited interaction\nwith those enforcing the Code of Conduct, is allowed during this period.\nViolating these terms may lead to a permanent ban.","ref":"code_of_conduct.html#3-temporary-ban"},{"type":"extras","title":"4. Permanent Ban - Code of Conduct","doc":"**Community Impact**: Demonstrating a pattern of violation of community\nstandards, including sustained inappropriate behavior, harassment of an\nindividual, or aggression toward or disparagement of classes of individuals.\n\n**Consequence**: A permanent ban from any sort of public interaction within the\ncommunity.","ref":"code_of_conduct.html#4-permanent-ban"},{"type":"extras","title":"Attribution - Code of Conduct","doc":"This Code of Conduct is adapted from the [Contributor Covenant][homepage],\nversion 2.1, available at\n[https://www.contributor-covenant.org/version/2/1/code_of_conduct.html][v2.1].\n\nCommunity Impact Guidelines were inspired by\n[Mozilla's code of conduct enforcement ladder][Mozilla CoC].\n\nFor answers to common questions about this code of conduct, see the FAQ at\n[https://www.contributor-covenant.org/faq][FAQ]. Translations are available at\n[https://www.contributor-covenant.org/translations][translations].\n\n[homepage]: https://www.contributor-covenant.org\n[v2.1]: https://www.contributor-covenant.org/version/2/1/code_of_conduct.html\n[Mozilla CoC]: https://github.com/mozilla/diversity\n[FAQ]: https://www.contributor-covenant.org/faq\n[translations]: https://www.contributor-covenant.org/translations","ref":"code_of_conduct.html#attribution"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}